@@ -1,9 +1,10 @@
-use std::io::Error;
+use std::io;
 use std::net::IpAddr;
 use std::str::FromStr;
-use std::sync::Arc;
+use std::sync::{Arc, Mutex};
 use std::time::Duration;
 
+use awak::net::{TcpListener, TcpStream};
 use shadowsocks::args::parse_args;
 use shadowsocks::cipher::Cipher;
 use shadowsocks::config::Config;
@@ -13,32 +14,26 @@ use shadowsocks::socks5::{
     v5::{TYPE_DOMAIN, TYPE_IPV4, TYPE_IPV6},
 };
 
-use awak::net::{TcpListener, TcpStream};
-use futures_util::FutureExt;
-use parking_lot::Mutex;
-
-fn main() -> anyhow::Result<()> {
+fn main() -> io::Result<()> {
     env_logger::init();
     let config = parse_args("sslocal").unwrap();
     log::info!("{}", serde_json::to_string_pretty(&config).unwrap());
-
     awak::block_on(async {
         let listener = TcpListener::bind(&config.local_addr).await?;
         log::info!("listening connections on {}", config.local_addr);
-
         let cipher = Cipher::new(&config.method, &config.password);
         let config = Arc::new(config);
-
         loop {
             let (socket, addr) = listener.accept().await?;
             let _ = socket.set_nodelay(true);
             log::debug!("accept tcp stream from addr {:?}", addr);
             let cipher = Arc::new(Mutex::new(cipher.reset()));
-            let proxy = proxy(config.clone(), cipher, socket).map(|r| {
-                if let Err(e) = r {
+            let config = config.clone();
+            let proxy = async move {
+                if let Err(e) = proxy(config, cipher, socket).await {
                     log::error!("failed to proxy; error={}", e);
                 }
-            });
+            };
             awak::spawn(proxy).detach();
         }
     })
@@ -48,7 +43,7 @@ async fn proxy(
     config: Arc<Config>,
     cipher: Arc<Mutex<Cipher>>,
     mut socket1: TcpStream,
-) -> Result<(u64, u64), Error> {
+) -> io::Result<(u64, u64)> {
     let (host, port) = socks5::handshake(&mut socket1, Duration::from_secs(3)).await?;
 
     log::debug!("proxy to address: {}:{}", host, port);
@@ -59,10 +54,6 @@ async fn proxy(
     let rawaddr = generate_raw_addr(&host, port);
     write_all(cipher.clone(), &mut socket2, &rawaddr).await?;
 
-    let keepalive_period = config.keepalive_period;
-    socket1.set_keepalive(Some(Duration::from_secs(keepalive_period)))?;
-    socket2.set_keepalive(Some(Duration::from_secs(keepalive_period)))?;
-
     let (n1, n2) = copy_bidirectional(&mut socket1, &mut socket2, cipher).await?;
     log::debug!("proxy local => remote: {}, remote => local: {:?}", n1, n2);
     Ok((n1, n2))
