@@ -2,53 +2,38 @@
 use std::io;
 use std::net::{Ipv4Addr, Ipv6Addr, SocketAddr};
 use std::str;
-use std::sync::Arc;
-use std::time::Duration;
+use std::sync::{Arc, Mutex};
 
+use awak::net::{TcpListener, TcpStream};
 use shadowsocks::args::parse_args;
 use shadowsocks::cipher::Cipher;
-use shadowsocks::config::Config;
 use shadowsocks::io::{copy_bidirectional, read_exact};
 use shadowsocks::resolver::resolve;
 use shadowsocks::socks5::v5::{TYPE_DOMAIN, TYPE_IPV4, TYPE_IPV6};
 use shadowsocks::util::other;
 
-use awak::net::{TcpListener, TcpStream};
-use futures_util::FutureExt;
-use parking_lot::Mutex;
-
-fn main() -> anyhow::Result<()> {
+fn main() -> io::Result<()> {
     env_logger::init();
-
     let config = parse_args("ssserver").expect("invalid config");
     log::info!("{}", serde_json::to_string_pretty(&config).unwrap());
-
     let cipher = Cipher::new(&config.method, &config.password);
     awak::block_on(async {
         let listener = TcpListener::bind(&config.server_addr).await?;
-        let config = Arc::new(config);
-
         loop {
             let (socket, addr) = listener.accept().await?;
             log::debug!("accept stream from addr {:?}", addr);
             let cipher = Arc::new(Mutex::new(cipher.reset()));
-            let proxy = proxy(config.clone(), cipher, socket).map(|r| {
-                if let Err(e) = r {
+            let proxy = async move {
+                if let Err(e) = proxy(cipher, socket).await {
                     log::error!("failed to proxy; error={}", e);
-                }
-            });
-
-            let task = awak::spawn(proxy);
-            task.detach();
+                };
+            };
+            awak::spawn(proxy).detach();
         }
     })
 }
 
-async fn proxy(
-    config: Arc<Config>,
-    cipher: Arc<Mutex<Cipher>>,
-    mut socket1: TcpStream,
-) -> io::Result<(u64, u64)> {
+async fn proxy(cipher: Arc<Mutex<Cipher>>, mut socket1: TcpStream) -> io::Result<(u64, u64)> {
     let (host, port) = get_addr_info(cipher.clone(), &mut socket1).await?;
     log::debug!("proxy to address: {}:{}", host, port);
 
@@ -59,10 +44,6 @@ async fn proxy(
     let _ = socket2.set_nodelay(true);
     log::debug!("connected to addr {}:{}", addr, port);
 
-    let keepalive_period = config.keepalive_period;
-    socket1.set_keepalive(Some(Duration::from_secs(keepalive_period)))?;
-    socket2.set_keepalive(Some(Duration::from_secs(keepalive_period)))?;
-
     let (n1, n2) = copy_bidirectional(&mut socket2, &mut socket1, cipher).await?;
     log::debug!("proxy local => remote: {}, remote => local: {}", n1, n2);
     Ok((n1, n2))
