@@ -1,3 +1,3 @@
-pub use crate::copy::{decrypt_copy, encrypt_copy};
+pub use crate::copy::copy_bidirectional;
 pub use crate::read_exact::read_exact;
 pub use crate::write_all::write_all;
