@@ -2,13 +2,12 @@ use std::io;
 use std::net::IpAddr;
 use std::str::FromStr;
 
-use crate::util::other;
-
-use c_ares_resolver::FutureResolver;
+use dns_resolver::Resolver;
 use once_cell::sync::Lazy;
 
-static GLOBAL_RESOLVER: Lazy<FutureResolver> =
-    Lazy::new(|| FutureResolver::new().expect("new FutureResolver error"));
+use crate::util::other;
+
+static GLOBAL_RESOLVER: Lazy<Resolver> = Lazy::new(Resolver::new);
 
 pub async fn resolve(host: &str) -> io::Result<IpAddr> {
     if let Ok(addr) = IpAddr::from_str(host) {
@@ -16,12 +15,12 @@ pub async fn resolve(host: &str) -> io::Result<IpAddr> {
     }
 
     let results = GLOBAL_RESOLVER
-        .query_a(host)
+        .lookup_host(host)
         .await
         .map_err(|e| other(&e.to_string()))?;
 
-    if let Some(result) = results.iter().next() {
-        return Ok(IpAddr::V4(result.ipv4()));
+    if !results.is_empty() {
+        return Ok(results[0]);
     }
 
     Err(other("resolve fail"))
